import {ProductGamesystem} from "../../../../src/app/project/game-model/gamesystems/ProductGamesystem";
import {SimpleGamesystem} from "../../../../src/app/project/game-model/gamesystems/SimpleGamesystem";
import {ProductState} from "../../../../src/app/project/game-model/gamesystems/states/ProductState";
import {ProductTransition} from "../../../../src/app/project/game-model/gamesystems/transitions/ProductTransition";

export class ProductStateTrainer {
  static INNERSTATE_LETTER_1 = "A";
  static INNERSTATE_LETTER_2 = "B";
  static INNERSTATE_LETTER_3 = "C";
  static INNERSTATE_NUMBER_1 = "1";
  static INNERSTATE_NUMBER_2 = "2";
  static INNERSTATE_NUMBER_3 = "3";

  static LETTERS_GAMESYSTEM_NAME = "Letters";
  static NUMBERS_GAMESYSTEM_NAME = "Numbers";
  static PRODUCT_GAMESYSTEM_NAME = "Product Gamesystem";

  static givenFullProductGamesystemWithTwoStates() {
    const letter_Gamesystem = new SimpleGamesystem(this.LETTERS_GAMESYSTEM_NAME, "");
    const letter_1 = letter_Gamesystem.createState(this.INNERSTATE_LETTER_1, "")!;
    const letter_2 = letter_Gamesystem.createState(this.INNERSTATE_LETTER_2, "")!;
    const number_gamesystem = new SimpleGamesystem(this.NUMBERS_GAMESYSTEM_NAME, "");
    const number_1 = number_gamesystem.createState(this.INNERSTATE_NUMBER_1, "")!;
    const number_2 = number_gamesystem.createState(this.INNERSTATE_NUMBER_2, "")!;
    const productGamesystem = new ProductGamesystem(this.PRODUCT_GAMESYSTEM_NAME, "");

    productGamesystem.states.push(new ProductState(  [letter_1, number_1]));
    productGamesystem.states.push(new ProductState(  [letter_1, number_2]));
    productGamesystem.states.push(new ProductState(  [letter_2, number_1]));
    productGamesystem.states.push(new ProductState(  [letter_2, number_2]));

    productGamesystem.transitions.push(new ProductTransition(productGamesystem.states[0], productGamesystem.states[1]))
    productGamesystem.transitions.push(new ProductTransition(productGamesystem.states[0], productGamesystem.states[2]))
    productGamesystem.transitions.push(new ProductTransition(productGamesystem.states[0], productGamesystem.states[3]))

    productGamesystem.innerGamesystems.push(letter_Gamesystem);
    productGamesystem.innerGamesystems.push(number_gamesystem);

    return productGamesystem;

  }

  static givenUnusedInnerStates(productGamesystem: ProductGamesystem) {
    const letterGamesystem = productGamesystem.findChildSystemByName(ProductStateTrainer.LETTERS_GAMESYSTEM_NAME) as SimpleGamesystem;
    const numberGamesystem = productGamesystem.findChildSystemByName(ProductStateTrainer.NUMBERS_GAMESYSTEM_NAME) as SimpleGamesystem;

    const letterGamesystemState =  letterGamesystem.createState(ProductStateTrainer.INNERSTATE_LETTER_3, "")!;
    const numberGamesystemState = numberGamesystem.createState(ProductStateTrainer.INNERSTATE_NUMBER_3, "")!;

    return [letterGamesystemState, numberGamesystemState];
  }

}
