import {ProductSystemGenerator} from "./ProductSystemGenerator";
import {ProductGenerationData} from "./ProductGenerationData";
import {ProductGeneratorResult} from "./ProductGeneratorResult";
import {ProductState} from "../states/ProductState";

export class IsolatedProductStateGenerator extends ProductSystemGenerator {

  generateIsolatedProductStates() {
    if(this.productGamesystem.innerGamesystems.length < 2) return;

    const leftInitialData = this.prepareChildsystemForGeneration(this.productGamesystem.innerGamesystems[0])
    const rightInitialData = this.prepareChildsystemForGeneration(this.productGamesystem.innerGamesystems[1])

    const initialGenerationResult = this.generateBinaryIsolatedProductStates(leftInitialData, rightInitialData);

    if(this.productGamesystem.innerGamesystems.length > 2) {
      for(let i=2; i<this.productGamesystem.innerGamesystems.length; i++) {
        const leftData = initialGenerationResult.productGenerationData;
        const rightData = this.prepareChildsystemForGeneration(this.productGamesystem.innerGamesystems[i]);

        const generationResult = this.generateBinaryIsolatedProductStates(leftData, rightData);
        this.assignGeneratedStatesAndTransitions(generationResult)
      }
    } else {
      this.assignGeneratedStatesAndTransitions(initialGenerationResult)
    }
  }


  protected assignGeneratedStatesAndTransitions(generationResult: ProductGeneratorResult) {
    this.productGamesystem.states = this.productGamesystem.states.concat(generationResult.states);
  }

  protected generateBinaryIsolatedProductStates(leftSystemData: ProductGenerationData, rightSystemData: ProductGenerationData): ProductGeneratorResult {
    const generatedProductStates: ProductState[] = []
    leftSystemData.states.forEach(leftState => {
      if(leftState.outgoingTransitions.length == 0 && leftState.incomingTransitions.length == 0) {
        rightSystemData.states.forEach(rightState => {
          if(rightState.outgoingTransitions.length == 0 && rightState.incomingTransitions.length == 0) {
            const leftConditions = this.getStateConditions(leftState, true);
            const rightConditions = this.getStateConditions(rightState, true);

            if(!this.contradictCombinedConditions(leftConditions, rightConditions)) {
              this.generateBinaryProductState(leftState, rightState, generatedProductStates);
            }
          }
        })
      }
    })
    return new ProductGeneratorResult(generatedProductStates, []);
  }

}
